@font-face {
	font-family: "Abadi MT Condensed Light";
    font-style:  normal;
    font-weight: normal;
    src: url(fonts/ABADIMT0.eot);
}

h1 {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/header.png', sizingMethod='crop');
	background-image: none;
	zoom: 1;
}

/* @group Steps */

#steps {
	position: absolute;
	left: 180px;
	background-image: none;
}

/* @group Tabs */

/* @group Dimmed */

#steps li#step-1 {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/step-1-dimmed.png', sizingMethod='crop');
}

#steps li#step-2 {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/step-2-dimmed.png', sizingMethod='crop');
}

#steps li#step-3 {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/step-3-dimmed.png', sizingMethod='crop');
}

#steps li#step-4 {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/step-4-dimmed.png', sizingMethod='crop');
}

#steps li#step-5 {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/step-5-dimmed.png', sizingMethod='crop');
}

/* @end */

#steps li.complete {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/tab-connector.png', sizingMethod='crop') !important;
	background-image: none !important;
}

#steps li.complete.current {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/tab-current.png', sizingMethod='crop') !important;
	background-image: none !important;
}

#steps li.complete#step-1 {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/tab-connector-left.png', sizingMethod='crop') !important;
	background-image: none !important;
}

#steps li.complete.current#step-1 {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/tab-current-left.png', sizingMethod='crop') !important;
	background-image: none !important;
}

/* @group Icons */

/* @group Normal */

#steps li#step-1 a {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/step-1-inactive.png', sizingMethod='crop');
}

#steps li#step-2 a {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/step-2-inactive.png', sizingMethod='crop');
}

#steps li#step-3 a {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/step-3-inactive.png', sizingMethod='crop');
}

#steps li#step-4 a {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/step-4-inactive.png', sizingMethod='crop');
}

#steps li#step-5 a {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/step-5-inactive.png', sizingMethod='crop');
}

/* @end */

/* @group Complete */

#steps li#step-1.complete a,
#steps li#step-1.current a {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/step-1-active.png', sizingMethod='crop');

}

#steps li#step-2.complete a,
#steps li#step-2.current a {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/step-2-active.png', sizingMethod='crop');
}

#steps li#step-3.complete a,
#steps li#step-3.current a {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/step-3-active.png', sizingMethod='crop');
}

#steps li.complete#step-4 a,
#steps li.current#step-4 a {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/step-4-active.png', sizingMethod='crop');
}

#steps li.complete#step-5 a,
#steps li.current#step-5 a {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/tabs-ie6/step-5-active.png', sizingMethod='crop');
}

/* @end */

#steps li a {
	background: none !important;
}

/* @end */



/* @end */

/* @group Tree */

#steps li ol
 {

}

#steps li.complete.current ol li,
#steps li.complete.current ol li a {
	filter: none !important;
}

#steps li li.active,
#steps li li.current {
	border-top: none #322f2e;
	border-bottom: none #82807e;
	height: auto;
	background: url(ie-steps-deep.png) repeat-y 0 bottom !important;
	padding-top: 1px;
	overflow: hidden;
}

#steps li li.active>a,
#steps li li.current>a {
	background: url(ie-steps-deep.png) repeat-y 0 top !important;	
}

#steps li li li.active,
#steps li li li.current {
	border-top: none #322f2e;
	border-bottom: none #82807e;
	height: auto;
	background: url(ie-steps-deeper.png) repeat-y 0 bottom !important;
	padding-top: 1px;
}

#steps li li li.active a,
#steps li li li.current a {
	background: url(ie-steps-deeper.png) repeat-y 0 top !important;	
}

#steps li.complete.current li ol.microns li {
	background-image: url(microns.gif) !important;
}

#steps li.complete.current li ol.microns li a {
	background-image: url(microns.gif) !important;
}

#steps li.complete.current li ol.microns li.current {
	background-position: -64px 0 !important;
}

#steps li.complete.current li ol.microns li.current.postponed {
	background-position: -80px 0 !important;
}

#steps li.complete.current li ol.microns li.current.risk {
	background-position: -96px 0 !important;
}



/* @end */



/* @end */

button.small,
.button-small {
	padding-top: 4px;
	padding-bottom: 1px;
}

#appendix {
	margin-right: -50px;
}

/* @group Global Navigation
 */

#navigation li
 {
	float: left;
}

#navigation li a
 {
	float: left;
	margin-right: 12px;
	padding-right: 0;
	padding-bottom: 1px;
	line-height: 22px;
	padding-top: 0;
}

#navigation li#home {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/logo.png', sizingMethod='crop');
	background: none;
}

#navigation li#home a {
	background-image: none;
	margin-right: 0;
	font: normal 14px/15px "Abadi-CL";

}



/* @end */

/* The next line prevents IE6 from making the ul too wide and to position it underneath p.question */

.radioList {
	width: 50%;
	margin-left: 0;
}

.radioList li label input {
	margin-top: -2px;
}

button {
	width: auto;
	overflow: visible;
}

.comprehensive,
.comprehensive fieldset,
.concise fieldset.comprehensive,
.compact fieldset.comprehensive {
	margin-bottom: 10px;
	float: left;
	display: inline;

}

legend
 {
	display: none;
}

fieldset.comprehensive p.question,
form.comprehensive fieldset p.question,
form.concise fieldset.comprehensive p.question,
form.compact fieldset.comprehensive p.question {
	display: inline;
	float: left;
	margin-left: -50%;
}

fieldset.concise,
form.concise fieldset {
	float: left;
	display: inline;
}

.icon {
	display: inline-block;
	width: 16px;
	height: 16px;
	text-indent: -1000px;
	background-image: url(icons.png);
	margin-top: -2px;
	position: relative;
	padding: 0;
	overflow: hidden;
	top: -2px;
}

/* @group Accordion */

dl.accordion dt {
	padding-bottom: 0;
	float: left;
	overflow: hidden;
	height: 25px;
}

dl.accordion dt button {
	position: relative;
	top: -17px;
	visibility: hidden;
}

dl.accordion dt:hover button,
dl.accordion dt.ui-state-active button {
	visibility: visible;
}

dl.accordion dd {
	width: 100%;
	overflow: hidden;
}

dl.accordion .colborder .superImpose {
	left: 0;
}

dl.accordion .colborder textarea {
}

dl.accordion .colborder hr{
	display: block;
	overflow: hidden;
	height: 0;
	margin: 0;
	border-style: none;
}

/* @end */

/* @group Comments */

#comments textarea {
	display: block;
	width: 99%;
}



/* @end */


.countries li {
	overflow: hidden;
}

dl.status .range {
	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='http://euphorie.hosting.simplon.biz/client/++resource++euphorie.style/graph-bars.png', sizingMethod='crop');
	background-image: none;
}
